
//#include "dataset.h"
#include "arch.h"
#include "error.h"

#include <string.h>
#include <stream.h>
#include <stdlib.h>

Dataset::Dataset(char *name, Datasettype Type, int size, int bytes, int floats,
		int opers):
	type(Type), totalsize(size), bytesperunit(bytes), 
	floatsperunit(floats), operations(opers), next(0) {
    this->name = new char [strlen(name) + 1] ;
    strcpy(this->name, name) ;
}

Dataset::~Dataset() {
    delete name ;
}

void Dataset::append(Dataset * data) {
    Dataset * d = this ;
    while (d->next != 0) d = d->next ;
    d->next = data ;
}

Dataset* Dataset::locate(char * name) {
    Dataset* p = this ;

    do {
	if (p->name != NULL)		//    test p->name != 0
	if (!strcmp(p->name, name)) return p ;
	p = p->next ;
    } while (p) ;
    return 0 ;
}

long Dataset::getbytes() {
    return totalsize * bytesperunit ;
}

long Dataset::getoperations() {
    return totalsize * bytesperunit * operations ;
}

void Dataset::display() {
    cout << name << " " << type << " " << totalsize ;
    cout << " " << bytesperunit << " " << floatsperunit << endl ;
}

float Dataset::transmit(Nodeset& nodeset, char* begin, char* end) {
    Nodeset* node = nodeset.locate(begin) ;
    if (!node) {
	cerr << "No such Nodeset : " << begin << end ;
	exit(-1) ;
    }

    Linkset* link = node->getlink(end) ;
    if ( !link ) {
	cerr << "Sorry, can't find nodeset : " << end << " through " ;
	cerr << "linkset " << endl ;
	exit(-1) ;
    }
//    link->display() ;

    if ( !(node = nodeset.locate(end)) ) error(end) ;

    return link->send(this, node->getgrainsize()) ;
}

float Dataset::datamove(Nodeset& nodeset, char* begin, char* end) {
    Nodeset* node = nodeset.locate(begin) ;
    if (!node) {
        cerr << "No such Nodeset : " << begin << end ;
        exit(-1) ;
    }

    Linkset* link = node->getlink(end) ;
//    link->display() ;

    return link->move(this) ;
}


float Dataset::update(Nodeset& nodeset, char* name) {
    Nodeset* node = nodeset.locate(name) ;
    if ( !node ) error(name) ;
    
    return node->calculate(this) ;
}

void Dataset::distribute(Distype distype, int procs_num) {
    if (!procs_num) {
	cerr << "Sorry, there's no CPU nodeset." << endl ;
	exit(-1) ;
    }

    switch (distype) {
	case Block1dim:
	case Block2dim:
	case Block3dim:
//cout << "ok" << endl;
		break ;
    }
}

#ifdef TEST_dataset

main()
{
    Dataset x;
}

#endif
